function plotGraph(D, oneWay)
%% Plots a communications graph based on incidence matrix

    if nargin <= 1
        oneWay = 0;
    end

    % Sanitycheck on D
    if any(sum(D,1))
        error('Invalid D supplied')
    end


    % Compute Laplacian
    L = D*D';
    
    % how many nodes?
    N = size(L,1);
    
    % How many links?
    nLinks = size(D, 2);
    
    % Generate points on a circle to set as nodes
    R = 1;
    
    % Get angles for the nodes
    offset = -pi/N;
    nodeAngles = linspace(-pi/2 + offset, (3/2)*pi+offset, N+1);
    
    colors = {'blue', 'red', 'green', 'magenta', 'cyan', 'yellow'};
    
    % Plot the nodes
    %fig = figure(3); clf;
    
    % Disable the axes
    set(gca,'xcolor',[1 1 1]);
    set(gca,'ycolor',[1 1 1]);
    set(gca,'ytick',[]);
    set(gca,'xtick',[]);

    hold on;
    
    for node = 1:N
        r = 0.1;
        
        nodePos = [R*cos(nodeAngles(node));
                   R*sin(nodeAngles(node));];
        
        t = 0:0.01:2*pi;
        fill(nodePos(1) + r*cos(t), nodePos(2) + r*sin(t), [colors{node}]);
        a = plot(nodePos(1) + r*cos(t), nodePos(2) + r*sin(t), 'k');% [colors{node}]);
        set(a, 'LineWidth', 2)
        
        
        
        
        
        text(nodePos(1) - 1.1*r, nodePos(2) + 1.5*r*sign(nodePos(2)), sprintf('Node %d', node))
        
    end
    
    % Plot all the links
    

    for link = 1:nLinks
        start=1;
        stop=1;
        
        for node = 1:N
            if D(node,link) == 1
                stop = node;
            elseif D(node,link) == -1
                start = node;
            end
        end
        
        plotCommLine(nodeAngles, R, start, stop, link, oneWay);
        
    end
    
    axis equal
    
    %R = 1.2*R;   
    xlim([-R R]);
    ylim([-R R]);
    
    %title(sprintf('Nodemap \nBlack lines represent undirected links'))

end

function plotCommLine(nodeAngles, R, from, to, link, oneWay)
    % ARROW SETUP
    tip = 0.5;
    scale = 0.05;
    arrow = [0 0;
                -tip -1;
                0 -0.7;
                tip -1;
                0 0;]';   

    RotMatrix = @(angle)([cos(angle) -sin(angle); sin(angle) cos(angle)]);

    % Get start pos
    startPos = [R*cos(nodeAngles(from));
                   R*sin(nodeAngles(from));];
               
               
               
   stopPos = [R*cos(nodeAngles(to));
                   R*sin(nodeAngles(to));];
    
   
   diffAngle = atan2(stopPos(2)-startPos(2), stopPos(1) - startPos(1));

   % augment
   r=0.1;
   startPos = [r*cos(diffAngle); r*sin(diffAngle)] + startPos;
   stopPos = -[r*cos(diffAngle); r*sin(diffAngle)] + stopPos;
   % Concat
   linePos = [startPos stopPos];
   
   % Plot the actual line
   a = line(linePos(1,:), linePos(2,:));
   set(a, 'Color', 'black')
   %set(a, 'LineStyle', '--')
   
   
   
   % Plot the arrowhead to Stop          
    target = RotMatrix(diffAngle-pi/2)*scale*arrow;
    plot(stopPos(1)+target(1,:),stopPos(2)+target(2,:),'k');
    patch(stopPos(1)+target(1,:), stopPos(2)+target(2,:), 'k')

   
if ~oneWay
   % Plot the arrowhead to start        
    target = RotMatrix(diffAngle+pi/2)*scale*arrow;
    plot(startPos(1)+target(1,:),startPos(2)+target(2,:),'k');
    patch(startPos(1)+target(1,:), startPos(2)+target(2,:), 'k')
end   
   % Plot textual line number
   gain = 1;
   if from < to
       gain = -1;
   end
   
   % Augment linePos to make the start-position for the text more 
   linePos = [startPos linePos];
   
   text(mean(linePos(1,:)), mean(linePos(2,:))+0.1*R*gain, sprintf('Link %d', link))
   
   
end

