/*
 * datapoint.cpp
 *
 *  Created on: Feb 8, 2013
 *      Author: krisklau
 */

#include "datapoint.h"


#include <mqueue.h>
#include <fcntl.h> // for definition of O_NONBLOCK
#include <errno.h>
#include <string.h>


// Used for request
#include <sys/select.h>
#include <sys/time.h>



#include <iostream>
using std::cerr;
using std::endl;
using std::cout;

DataPoint::DataPoint(DATAPOINT_TYPE type, string queueName, int msgSize):
	type(type),
	queueName(queueName),
	msgSize(msgSize) {

}

int DataPoint::open(void) {
	int flags = O_CREAT;// | O_NONBLOCK;
	switch(this->type) {
	case DP_READ:
		flags |= O_RDONLY;
		break;
	case DP_WRITE:
		flags |= O_WRONLY;
		break;
	default:
		cerr << "Invalid type setting." << endl;
		break;
	}


	int perms = S_IRUSR | S_IWUSR;

	/* Set up attributes	 */
	struct mq_attr attr;


	attr.mq_maxmsg = QUEUE_MAXMSG;  // 10 is max hard-coded in OS. Must be increased manually on OS-level if this is raised beond 10.
	attr.mq_msgsize = this->msgSize;


	// Open the queue!

	this->mqDescriptor = mq_open(queueName.c_str(), flags, perms, &attr);

	if(this->mqDescriptor == (mqd_t) -1) {
		// Could not create the queue. For now, exit and print error.
		cerr << "Error in creating queue. " <<  strerror(errno) << endl;
		return -1;
	}
	else {
		cout << "Que " << this->queueName<< " created successfylly." << endl;
		cout << "That is, ID: : " << this->mqDescriptor << endl;
	}

	return 0;

}

int DataPoint::close(void) {
	mq_close(this->mqDescriptor);

	// Maybe add an unlink here also.
	//mq_unlink(this->queueName.c_str());

	return 0;
}

mqd_t DataPoint::get_mqd(void) {
	return this->mqDescriptor;
}

int DataPoint::get_fd(void) {
	return this->mqDescriptor;
}

int DataPoint::send(void* msg) {
	int priority = 5;

	// Consider changing for select

	if(this->numberOfMessages() >= QUEUE_MAXMSG) {
		cerr << "Que maxed out. Que: " << this->queueName << endl;
		return -1;
	}

	if( mq_send(this->mqDescriptor, (char*) msg, this->msgSize, priority) < 0) {
		cerr << "Datapoint: Error sending message: " << __FILE__ << this->queueName <<  strerror(errno) << endl;
		return -1;
	}
	return 0;
}

int DataPoint::recieve(void* msg) {
	// NB; will block, so user is adviced to use select!

	if(mq_receive(this->mqDescriptor, (char*) msg, this->msgSize, NULL) < 0) {
		if(errno == EAGAIN) cerr << "EAGAIN.." << endl;
		cerr <<  "Error receiving message: " <<  strerror(errno) << endl;
		return -1;
	}

	return 0;
}

int DataPoint::numberOfMessages(void) {
	// Gets the number of messages in the queue



	struct mq_attr attr;
	if(mq_getattr(this->mqDescriptor, &attr) < 0) {

		cerr << "Unable to read attributes of que: " << errno  << strerror(errno) << endl;
		return -1;
	}

	return attr.mq_curmsgs;



}

int DataPoint::get_msgSize(void) {
	return this->msgSize;
}




DataPointRequest::DataPointRequest(DATAPOINT_TYPE dataType,
		DATAPOINT_TYPE requestType, string queueName, int msgSize,
		string requestQueueName,
		int requestMsgSize) :
		dataPoint(dataType, queueName, msgSize),
		requestPoint(requestType, requestQueueName, requestMsgSize),
		timeout_us(1E5){
	/* Intentionally empty */
}

int DataPointRequest::open(void) {
	// Open both queues.

	if(this->dataPoint.open() < 0) {
		cerr << "DP_request: Failed to open data line. " << endl;
		return -1;
	}
	if(this->requestPoint.open() < 0) {
		cerr << "DP_request: Failed to open request line. " << endl;
		return -1;
	}
	return -1;


}

/* Somewhat main workhorse method */
int DataPointRequest::request(void* msg, void* rq_msg) {


	// Send the request

	if(this->requestPoint.send(rq_msg) < 0) {
		cerr << "DP_request: Error sending the request" << endl;
		return -1;
	}

	// Set up for select, wait for response on DATA line
	fd_set descriptorSet;
	struct timeval tv;

	FD_ZERO(&descriptorSet);

	FD_SET(this->dataPoint.get_fd(), &descriptorSet);

	// Setup timeout
	tv.tv_sec = 0;
	tv.tv_usec = this->timeout_us;

	/*
	 * Start select
	 */

	int retval = select(this->dataPoint.get_fd()+1, &descriptorSet, NULL, NULL, &tv);

	if (retval == -1) {
		cerr << "DP_request: Error using select when sending request. " << endl;
		return -1;
	}
	else if(retval) { // Positive, means we got a select statement
		if (FD_ISSET(this->dataPoint.get_fd(), &descriptorSet)) {
			// Message return

			// are there multiple messages?
			int nMsg = this->dataPoint.numberOfMessages();
			if(nMsg > 1) {
				cerr << "DP_request: Multiple data messages detected. Discarding. " << endl;
			}
			for(int i = 0; i<nMsg; i++) {

				if(this->dataPoint.recieve(msg) < 0) {
					cerr << "DP_request: Error receiving data message." << endl;
					return -1;
				}
			}
			// valid message at this point
			return 0;
		}
		else {
			cerr << "Invalid select out return." << endl;
			return -1;
		}

	}
	else { // timeout
		cerr << "DP_request: Timeout on data return" << endl;
		return -1;
	}


}

int DataPointRequest::request(void* msg, DP_request_msg* rq_msg) {
	// Cast to void
	return this->request(msg, (void*) rq_msg);
}




int DataPointRequest::request(void* msg) {
	// This methods sends a request, waits response with select
	// Can timeout

	/*
	 * SETUP
	 */

	// Set up a request
	DP_request_msg rq_msg;
	rq_msg.request = 1;

	// Send this request
	return this->request(msg, &rq_msg);


}

int DataPointRequest::get_request_fd(void) {
	return this->requestPoint.get_fd();
}

int DataPointRequest::read_request(void* msg) {

	// check for multiple messages
	int nRequests = this->requestPoint.numberOfMessages();
	if(nRequests == 0)
		cerr << "No messages.. NO RETURN! " << endl;
	if(nRequests > 1) {
		cerr << "DP_request: Multiple requests detected. Discarding" << endl;
	}
	for(int i = 0; i < nRequests; i++) {
		if(this->requestPoint.recieve(msg) < 0) {
			cerr << "DP_request: Error receiving request message" << endl;
			return -1;
		}
	}
	return 0;

}


int DataPointRequest::read_request(DP_request_msg* msg) {
	// Cast to void, read.
	return this->read_request((void*) msg);
}

int DataPointRequest::send_data(void* msg) {
	// simply send the data
	if(this->dataPoint.send(msg) < 0) {
		cerr << "DP_request: Error sending data" << endl;
		return -1;
	}
	return 0;
}

void DataPointRequest::forceClose(void) {
	this->dataPoint.close();
	this->requestPoint.close();
}

void DataPointRequest::set_timeout(int timeout_us) {
	this->timeout_us = timeout_us;
}

DataPointRequestHost::DataPointRequestHost(string queueName, int msgSize,
		string requestQueueName,
		int requestMsgSize) :
		DataPointRequest(DP_WRITE, DP_READ, queueName, msgSize, requestQueueName, requestMsgSize){
	/* Intentionally empty */
}

DataPointRequestClient::DataPointRequestClient(string queueName, int msgSize,
		string requestQueueName,
		int requestMsgSize) :
		DataPointRequest(DP_READ, DP_WRITE, queueName, msgSize, requestQueueName, requestMsgSize) {
	/* Intentionally empty */
}











