/*
 * i2cdevice.cpp
 *
 *  Created on: Feb 4, 2013
 *      Author: krisklau
 */

#include "i2cdevice.h"
#include <iostream>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <math.h> // for atan2
#include <limits> // For min-max int limits

#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/i2c-dev.h>

#define I2C_PATH "/dev/i2c-%d"
#define I2C_PATH_MAXLENGTH 20

using std::cerr;
using std::endl;

I2CDevice::I2CDevice(I2C_LINE line, char address) {
	// TODO Auto-generated constructor stub

	switch(line) {
	case I2C_1:
		this->line = 1;
		break;
	case I2C_2:
		this->line = 3;
		cerr << "Currently i2C line 2 is not for use. Setting to line 3. This should exit the program.." << endl;
		break;
	case I2C_3:
		this->line = 3;
		break;
	default:
		this->line = 3;
		cerr << "Invalid line setting. Set to 3" << endl;
		break;
	}

	this->address = address;

}

int I2CDevice::writeRegister(char registerAddress, char* data, int length) {
	// First, open the device

	char path[I2C_PATH_MAXLENGTH];
	snprintf(path, I2C_PATH_MAXLENGTH, I2C_PATH, this->line);

	int file = open(path, O_RDWR);

	if(file < 0) {
		cerr << "Unable to open i2c-device" << endl;
		close(file);
		return -1;
	}

	// Try io interface
	if(ioctl(file, I2C_SLAVE, this->address) < 0) {
		cerr << "Unable to set up ioctl." << endl;
		close(file);
		return -1;
	}

	// Write contents
	char* writeBuffer = (char*)malloc(length+1);
	writeBuffer[0] = registerAddress;
	memcpy(&writeBuffer[1], data, length); // remember to free!




	if(write(file, writeBuffer, length+1) != length+1) {
		cerr << "I2C: error writing to device, at " << registerAddress <<", value: " << data[0] << endl;
		close(file);
		return -1;
	}

	// Done. Free
	free(writeBuffer);

	// Close file
	close(file);
	return 0;



}

int I2CDevice::writeRegister(char registerAddress, char data) {
	return this->writeRegister(registerAddress, &data, 1);
}

int I2CDevice::readRegister(char registerAddress, char* buffer, int length) {
	// First, open the device

	char path[I2C_PATH_MAXLENGTH];
	snprintf(path, I2C_PATH_MAXLENGTH, I2C_PATH, this->line);

	int file = open(path, O_RDWR);

	if(file < 0) {
		cerr << "Unable to open i2c-device" << endl;
		close(file);
		return -1;
	}

	// Try io interface
	if(ioctl(file, I2C_SLAVE, this->address) < 0) {
		cerr << "Unable to set up ioctl." << endl;
		close(file);
		return -1;
	}

	// Set to correct address
	if(write(file, &registerAddress, 1) != 1) {
		cerr << "Unable to request from register" << endl;
		close(file);
		return -1;
	}

	// Read the data
	int n = read(file, buffer, length);
	if(n < 0) {
		cerr << "Unable to read data. " << endl;
		close(file);
		return -1;
	}

	// Data is read. Close and exit
	close(file);
	return 0;

}

Magnetometer::Magnetometer(char address):
		I2CDevice(I2C_3, address),
		scale_x(1),
		scale_y(1),
		scale_z(1),
		allMinMax({0}),
		isCalibrating(false){
	// Set bias to 0
	this->setBias(0,0,0);

}

int Magnetometer::setBias(int x, int y, int z) {
	this->bias_x = x;
	this->bias_y = y;
	this->bias_z = z;

	return 0;
}

int Magnetometer::calibrationStart(void) {
	// Set is calibrating
	this->isCalibrating = true;

	// Reset bias
	this->setBias(0,0,0);



	// Reset internal calibration buffer
	for(int i = 0; i < 3; i=i+1) {
		this->allMinMax[2*i]   = std::numeric_limits<int>::max();
		this->allMinMax[2*i+1] = std::numeric_limits<int>::min();
	}
	this->scale_x = 1;
	this->scale_y = 1;
	this->scale_z = 1;

	return 0;
}

int Magnetometer::calibrationStop(void) {
	// Shall read the maxes and mins. For now, just print.
	// But later, it will also set the bias values

	cerr << "Calib: ";
	for (int j = 0; j< 3; j = j+1) {
		cerr << " " << char('X'+j) << ": " << this->allMinMax[2*j] << " - " << this->allMinMax[2*j+1];
	}
	cerr << endl;

	// All should give -30 to 30

	// Test x first
	int diff = (allMinMax[1] - allMinMax[0]);
	int avg  = (allMinMax[1] + allMinMax[0])/2;

	this->bias_x = -avg;
	//this->scale_x = 200.0/diff;

	cerr << "Got bias X: " << this->bias_x << " and scale: " << this->scale_x << endl;

	diff = (allMinMax[3] - allMinMax[2]);
	avg = (allMinMax[3] + allMinMax[2])/2;

	this->bias_y = -avg;
	//this->scale_y = 200.0/diff;

	cerr << "Got bias Y: " << this->bias_y << " and scale: " << this->scale_y << endl;

	this->isCalibrating = false;

	return 0;

}


float Magnetometer::getAngle(void) {

	return 0;
}

int Magnetometer::getAngleDeg_int(void) {

	return int(RAD2DEG * atan2(this->y, this->x) + 0.5); // Add 0.5 for rounding

}




HMC5883L::HMC5883L():Magnetometer(0x1e) {
	// Set into continious mode
	if(this->writeRegister(0x02, 0x00) < 0) {
		cerr << "HMC5883L: Unable to set mode." << endl;
	}
}

int HMC5883L::read(void) {
	// Input buffer
	char inputBuffer[6] = {0};
	if(this->readRegister(0x03, inputBuffer, 6) < 0) {
		cerr << "HMC5883L: Unable to read data." << endl;
		return -1;
	}

	int16_t tmp;
	tmp = inputBuffer[0] << 8 | inputBuffer[1];
	this->x = tmp;

	tmp = inputBuffer[2] << 8 | inputBuffer[3];
	this->y = tmp;

	tmp = inputBuffer[4] << 8 | inputBuffer[5];
	this->z = tmp;

	// Set max and min values if calibrating
	if(this->isCalibrating) {
		allMinMax[0] = (this->x < allMinMax[0]) ? this->x : allMinMax[0];
		allMinMax[1] = (this->x > allMinMax[1]) ? this->x : allMinMax[1];
		allMinMax[2] = (this->y < allMinMax[2]) ? this->y : allMinMax[2];
		allMinMax[3] = (this->y > allMinMax[3]) ? this->y : allMinMax[3];
		allMinMax[4] = (this->z < allMinMax[4]) ? this->z : allMinMax[4];
		allMinMax[5] = (this->z > allMinMax[5]) ? this->z : allMinMax[5];
	}
	else {
		// apply bias and scale
		this->x = (this->x + this->bias_x) * this->scale_x;
		this->y = (this->y + this->bias_y) * this->scale_y;
		this->z = (this->z + this->bias_z) * this->scale_z;
	}

	return 0;
}




