/*
 * uart_channel.cpp
 *
 *  Created on: Feb 7, 2013
 *      Author: adamleon
 */

#include "uart_channel.h"

UARTChannel::UARTChannel() {
	this->is_configured = false;
	this->is_opened = false;
	this->fd = 0;

	if(DUMMY_MODE) {
		cout << "uart_channel: not an ARM device. dummy_mode activated" << endl;
	}
}

UARTChannel::~UARTChannel() {
	// TODO Auto-generated destructor stub
}

/**
 * Configures a channel
 * @param channel The channel to conigure to
 * @return 0 if the configuration was successful, else -1
 */
int UARTChannel::configure(UARTChannelType channel) {
	this->channel = channel;

	// If the code is running on the beaglebone
	if(!DUMMY_MODE) {
		std::string rx_filename = string();
		std::string tx_filename = string();
		std::string rx_mode = "0";
		std::string tx_mode = "0";

		// See which channel the input channel is
		// and set up the configuration file thereafter
		switch(this->channel)
		{
			case UART_1:
				this->port = "/dev/ttyO1";
				rx_filename = "/sys/kernel/debug/omap_mux/uart1_rxd";
				tx_filename = "/sys/kernel/debug/omap_mux/uart1_txd";
				rx_mode = "20";
				tx_mode = "0";
				break;
			case UART_2:
				this->port = "/dev/ttyO2";
				rx_filename = "/sys/kernel/debug/omap_mux/spi0_sclk";
				tx_filename = "/sys/kernel/debug/omap_mux/spi0_d0";
				rx_mode = "21";
				tx_mode = "1";
				break;
			case UART_4:
				this->port = "/dev/ttyO4";
				rx_filename = "/sys/kernel/debug/omap_mux/gpmc_wait0";
				tx_filename = "/sys/kernel/debug/omap_mux/gpmc_wpn";
				rx_mode = "26";
				tx_mode = "6";
				break;
			case UART_5:
				this->port = "/dev/ttyO5";
				rx_filename = "/sys/kernel/debug/omap_mux/lcd_data9";
				tx_filename = "/sys/kernel/debug/omap_mux/lcd_data8";
				rx_mode = "24";
				tx_mode = "4";
				break;
			default:
				cout << "uart_channel: configuration failed - uart " << this->channel << " does not exist" << endl;
				return -1;
		}

		FILE* uart_handler = NULL;

		// Configure the RX pin
		if((uart_handler = fopen(rx_filename.c_str(), "r+")) != NULL) {
			fwrite(rx_mode.c_str(), sizeof(char), rx_mode.size(), uart_handler);
			fclose(uart_handler);
		}
		else {
			cout << "uart_channel: unable to configure RX port" << endl;
			return -1;
		}

		// Configure the TX pin
		if((uart_handler = fopen(tx_filename.c_str(), "r+")) != NULL) {
			fwrite(tx_mode.c_str(), sizeof(char), tx_mode.size(), uart_handler);
			fclose(uart_handler);
		}
		else {
			cout << "uart_channel: unable to configure TX port" << endl;
			return -1;
		}
	}
	// If the code is running on pc
	else {
		// Set the channel to zero
		this->channel = (UARTChannelType)0;
		FILE* uart_handler = NULL;

		// Open the usb serial file to see which port the XBee is connected to
		if((uart_handler = fopen("/proc/tty/driver/usbserial", "r")) != NULL) {
			char port_number[256];

			// Go through each line
			while(fgets(port_number, sizeof(port_number), uart_handler) != NULL) {
				// Continue if the first character is not a number
				if(port_number[0] < '0' || port_number[0] > '9') {
					continue;
				}

				// Set the port of the UART port
				this->port = "/dev/ttyUSB";
				this->port += port_number[0];
				break;
			}

			// If the file does not contain a connection to the XBee
			if(this->port.empty()) {
				cout << "uart_channel: no UART channel found" << endl;
				return -1;
			}
		} else {
			cout << "uart_channel: unable to open usbserial: "  << strerror(errno) << endl;
			return -1;
		}
	}

	struct termios settings;

	// Open port
	int port_file = ::open(this->port.c_str(), O_RDWR);
	if(port_file == -1) {
		cout << "uart_channel: unable to open port - " << this->port << endl;
		return -1;
	}

	// Get the current settings
	tcgetattr(port_file, &settings);

	// Set the input/output speed
	cfsetispeed(&settings, B57600);
	cfsetospeed(&settings, B57600);

	// Make the channel canonial
	// and have end of file as EOT ASCII char
	//settings.c_lflag = ICANON;		// enable
	settings.c_lflag &= ~(ICANON); // disable
	settings.c_cc[VEOF] = 0x4;

	// Apply the new settings
	if(tcsetattr(port_file, TCSANOW, &settings) < 0) {
		cerr << "uart_channel: Unable to set configuration" << endl;
		return -1;
	}

	::close(port_file);
	// Finished configuration
	this->is_configured = true;
	cout << "uart_channel: configured channel " << this->channel << " to port " << this->port << endl;
	return 0;
}

/**
 * Writes to a channel
 * @param message The message to write
 * @return 0 if the write was successful, else -1
 */
int UARTChannel::write(string message) {
	// If the channel is not configured
	if(!this->is_configured) {
		cout << "uart_channel: write error - not configured" << endl;
		return -1;
	}

	// End of transmission
	char eot = 0x04;

	// Write to the channel
	int port_file;

	if(!this->is_opened) {
		 port_file = ::open(this->port.c_str(), O_WRONLY | O_NONBLOCK | O_FSYNC);
	}
	else {
		port_file = this->fd;
	}


	if(port_file < 0) {
		// Unable to write to the port
		cout << "uart_channel: unable to write to port " << this->port << endl;
		cerr << strerror(errno) << endl;
		return -1;
	}


	// Write the message
	size_t bytes_written = 0;
	while(bytes_written < message.size()) {
		int written = ::write(port_file, &message[0] + bytes_written, message.size()-bytes_written);
		if(written < 0) {
			cerr << "uart_channel: Error writing to file." << endl;
			cerr << strerror(errno) << endl;
			if(!this->is_opened) { ::close(port_file);}
			return -1;
		}
		// Increment written
		bytes_written += written;

	}
	// Write end of transmission
	bytes_written = 0;
	while(bytes_written < 1) {
		int written = ::write(port_file, &eot, 1);
		if(written < 0) {
			cerr << "uart_channel: Error writing eof." << endl;
			cerr << strerror(errno) << endl;
			if(!this->is_opened) { ::close(port_file);}
			return -1;
		}

		//increment (break)
		bytes_written += written;
	}
	if(!this->is_opened) { ::close(port_file);}
	return 0;



}

/**
 * Reads from the channel
 * @param message A string returning the message read
 * @return 0 if a message was read, else -1
 */
int UARTChannel::read(string& message) {
	// If the channel is not configured
	if(!this->is_configured) {
		cout << "uart_channel: read error - not configured" << endl;
		return -1;
	}

	// Parameters for the received message
	char buffer[256];
	int length;
	message = string();

	// Open the channel
	int port_file;
	if(!this->is_opened) {
		 port_file = ::open(this->port.c_str(), O_RDONLY | O_NONBLOCK);
	}
	else {
		port_file = this->fd;
	}


	if(port_file > 0) {
		if((length = ::read(port_file, buffer, 256)) >= 0) {
			// Set the message
			message = string(buffer, length);
		}

		if(!this->is_opened) { ::close(port_file);}
		return length;
	}

	// Unable to read from port
	cout << "uart_channel: unable to read from port " << this->port << endl;
	return -1;
}

int UARTChannel::open() {
	// Opens the shit
	if(!this->is_configured) {
		cout << "uart_channel: read error - not configured" << endl;
		return -1;
	}

	// Open the channel
	int port_file = ::open(this->port.c_str(), O_RDWR);// | O_NONBLOCK | O_FSYNC);
	if(port_file < 0) {
		cerr << "Error opening file for reading" << endl;
		return -1;
	}

	this->is_opened = true;
	this->fd = port_file;
	return 0;


}

int UARTChannel::close() {
	// just close..
	if(this->is_opened) {
		::close(this->fd);
		this->is_opened = false;
	}
	return 0;

}





