/*
 * main.cpp
 *
 *  Created on: Mar 4, 2013
 *      Author: adamleon
 */
#include "timer.h"
#include <math.h>
#include "uart_channel.h"
#include "camera_client.h"
#include "logger.h"
#include "analog_laser.h"
#include "particle.h"
#include "servo.h"

#include <vector>
#include <sstream>
#include <iostream>
#include <errno.h>
#include <string>
#include <sys/fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#define PI 3.141529
#define MAP_RADIUS 1.0
#define PARTICLE_COUNT 10000
#define MEASUREMENT_LENGTH 0.025
#define UPDATE_UNSERTAINTY 0.999
#define RADIUS_UNSERTAINTY 0.02
#define HEADING_UNSERTAINTY 0.1
using namespace std;

Particle particles[PARTICLE_COUNT];
float prev_x, prev_y;
void updateParticles(float, float);
void initializeParticles(float);
void resampleParticles();

int main(int argc, char** argv) {
	AnalogLaser laser(AIN0);

	cout << "laser_read: begin" << endl;

	Logger logger("laser_servo");
	logger.init();
	logger.start();
	// Timer info
	periodic_info timerInfo;

	Servo servoA(SERVO_0A);
	Servo servoB(SERVO_0B);
	Servo servoC(SERVO_1A);

	servoA.init();
	servoA.start();
	servoB.init();
	servoB.start();
	servoB.stop();
	servoC.init();
	servoC.start();
	servoC.stop();

	initializeParticles(0);

	int ser = 0, direction = 1;

	make_periodic(5E5, &timerInfo);
	while(1) {

		logger.timestamp();
		servoA.write(ser);
		ser += direction;
		if(ser >= 90) direction = -1;
		else if(ser < 0) break;
		//cout << ser << endl;
		logger.add("Servo", (double)ser);


		int array_size = 25;
		double mes[array_size];
		for(int i = 0; i < array_size; i++) {
			mes[i] = laser.measureDistance();
		}

		int i, j, temp;

		for (i = (array_size - 1); i > 0; i--)
		{
		    for (j = 1; j <= i; j++)
		    {
		    	if (mes[j-1] > mes[j])
				{
		    		temp = mes[j-1];
				    mes[j-1] = mes[j];
				    mes[j] = temp;
				}
			}
		}

		double m = 0;
		for(i = 0; i < 5; i++) {
			m += mes[10 + i];
		}

		m /= 5.0;

		//cout << m << endl;
		logger.add("Laser", m);
		logger.execTimestamp();
		logger.write();

		float ang = (float)ser * (float)PI / 180;
		updateParticles(ang, 0.01f * (float)m);
		resampleParticles();
		wait_period(&timerInfo);
	}

	Logger log("particles");
	log.init();
	log.start();
	for(int i = 0; i < PARTICLE_COUNT; i++)
	{
		log.timestamp();
		log.add("ID", (double)i);
		log.add("Weight", particles[i].getWeight());
		log.add("X", particles[i].getCoordinates().getX());
		log.add("Y", particles[i].getCoordinates().getY());
		log.write();
	}
	cout << "laser_read: end" << endl;
	return 0;
}

Coordinate getCoordinates(float angle, float measurement) {
	return Coordinate(cos(angle) * measurement, sin(angle) * measurement);
}

void setHeading(float heading) {
	for(int i = 0; i < PARTICLE_COUNT; i++) {
		particles[i].setHeading(heading);
	}
}

void setPosition(float x, float y) {
	for(int i = 0; i < PARTICLE_COUNT; i++) {
		Coordinate coords = particles[i].getCoordinates();
		coords.setX(coords.getX() + x-prev_x);
		coords.setY(coords.getY() + y-prev_y);

		prev_x = x;
		prev_y = y;
		if(coords.getLength() > MAP_RADIUS) {
			particles[i].reset();
		}
	}

	prev_x = x;
	prev_y = y;
}

void initializeParticles(float heading) {
	srand(time(NULL));

	for(int i = 0; i < PARTICLE_COUNT; i++)
	{
		double x = MAP_RADIUS;
		double y = MAP_RADIUS;
		while((x * x) + (y * y) > MAP_RADIUS * MAP_RADIUS) {
			x = ((double)rand() / RAND_MAX) * 2 * MAP_RADIUS;
			x -= MAP_RADIUS;
			y = ((double)rand() / RAND_MAX) * 2 * MAP_RADIUS;
			y -= MAP_RADIUS;
		}
		particles[i] = Particle((float)x, (float)y, heading);
	}
}

void updateParticles(float angle, float measurement) {
	for(int i = 0; i < PARTICLE_COUNT; i++) {
		// Update weights
		if(particles[i].getWeight() > 0.5)
		{
			particles[i].setWeight(particles[i].getWeight() * UPDATE_UNSERTAINTY);
		}
		else
		{
			particles[i].setWeight(particles[i].getWeight() * (2 - UPDATE_UNSERTAINTY));
		}
	}

	// Gets the AB vector
	Coordinate ab = getCoordinates(angle, measurement);

	// Gets the norm(AB)^2
	float atb2 = pow(ab.getX(),2) + pow(ab.getY(),2);

	for(int i = 0; i < PARTICLE_COUNT; i++) {
		// Gets the AP vector
		Coordinate ap = particles[i].getCoordinates();

		//cout << ap.getX() << " " << ap.getY() << endl;

		// Gets the AP dot AB
		float ap_dot_ab = ap.getX()*ab.getX() + ap.getY()*ab.getY();

		// Gets the ratio of the nearest point on the line
		float t = ap_dot_ab / atb2;

		if(t > 1) t = 1;
		else if (t < 0) t = 0;

		float x = ab.getX() * t;
		float y = ab.getY() * t;

		float length = (float)sqrt(((ap.getX() - x) * (ap.getX() - x)) + ((ap.getY() - y) * (ap.getY() - y)));
		length /= MEASUREMENT_LENGTH;
		if(length <= 1)
		{
			if(t < 0.95)
			{
				particles[i].setWeight(0.5f * length * length);
			}
			else
			{
				particles[i].setWeight(t);
			}
		}
	}
}

void resampleParticles() {
	float maxWeight = 0;
	for(int i = 0; i < PARTICLE_COUNT; i++) {
		if(particles[i].getWeight() > maxWeight) {
			maxWeight = particles[i].getWeight();
		}
	}

	int index = ((double)rand() / RAND_MAX) * PARTICLE_COUNT;
	double beta = 0.0;
	for(int i = 0; i < PARTICLE_COUNT; i++) {
		if(particles[i].getWeight() < 0.4) {
			if(rand() % 4)
			{
				beta += ((double)rand() / RAND_MAX) * 2;

				while(beta > particles[index].getWeight())
				{
					beta -= particles[index].getWeight();
					index = (index + 1) % PARTICLE_COUNT;
				}

				particles[i].setWeight(particles[index].getWeight());

				double radius_error = ((double)rand() / RAND_MAX) * (RADIUS_UNSERTAINTY);
				double theta_error = ((double)rand() / RAND_MAX) * 2 * PI;
				double heading_error = (((double)rand() / RAND_MAX) * 2 * HEADING_UNSERTAINTY) - HEADING_UNSERTAINTY;

				float coordX = particles[index].getCoordinates().getX() + (radius_error * cos(theta_error));
				float coordY = particles[index].getCoordinates().getY() + (radius_error * sin(theta_error));
				particles[i].setCoordinates(coordX, coordY);

				particles[i].setHeading(particles[index].getHeading() + heading_error);
			}
		}
	}
}

